@@ -2092,7 +2092,7 @@ EAction.getEntityId = function(di, action, event, preview, selectable) {
     }
 
     var altPressed = isAltPressed(event);
-    if (!altPressed || preview) {
+    if (!altPressed || preview || RSettings.getBoolValue("GraphicsView/DisableAltPicking")===true) {
         if (!isNull(action.idFromContextMenu)) {
             // user already chose an entity from the context menu:
             return action.idFromContextMenu;
@@ -2211,6 +2211,10 @@ EAction.warnNotLine = function() {
     EAction.handleUserWarning(qsTr("Entity is not a line."));
 };
 
+EAction.warnNotArc = function() {
+    EAction.handleUserWarning(qsTr("Entity is not an arc."));
+};
+
 EAction.warnNotArcCircle = function() {
     EAction.handleUserWarning(qsTr("Entity is not an arc or circle."));
 };
